import json

from .explorer import Explorer


def fetch_abi(explorer: Explorer, address: str):
    contract_info = explorer.contract.getsourcecode(address=address)[0]
    abi = json.loads(contract_info["ABI"])
    if contract_info["Proxy"] == "1":
        implementation_info = explorer.contract.getsourcecode(
            address=contract_info["Implementation"]
        )[0]
        abi = json.loads(implementation_info["ABI"])

    return abi



from web3 import Web3

from .explorer import (
    Explorer,
    etherscan,
    snowtrace,
    ftmscan,
    arbiscan,
    bscscan,
    optimistic_etherscan,
    polygonscan,
)


class Chain(Web3):
    explorer: Explorer | None


def chain_w3(rpc, explorer=None):
    w3 = Web3(Web3.HTTPProvider(rpc))
    w3.explorer = explorer
    return w3


ethereum = chain_w3("https://rpc.ankr.com/eth", etherscan)
avalanche = chain_w3("https://rpc.ankr.com/avalanche-c", snowtrace)
fantom = chain_w3("https://rpc.ankr.com/fantom", ftmscan)
arbitrum = chain_w3("https://rpc.ankr.com/arbitrum", arbiscan)
bsc = chain_w3("https://rpc.ankr.com/bsc", bscscan)
optimism = chain_w3("https://rpc.ankr.com/optimism", optimistic_etherscan)
polygon = chain_w3("https://rpc.ankr.com/polygon", polygonscan)
